@@ -1,4 +1,7 @@
+#ifndef BOOST_TEST_DYN_LINK
 #define BOOST_TEST_DYN_LINK
+#endif
+
 #include <boost/test/unit_test.hpp>
 
 #include "test-syncres_cc.hh"
@@ -884,68 +887,6 @@ BOOST_AUTO_TEST_CASE(test_nsec3_denial_too_many_iterations)
   BOOST_CHECK_EQUAL(denialState, dState::INSECURE);
 }
 
-BOOST_AUTO_TEST_CASE(test_nsec3_many_labels_between_name_and_closest_encloser)
-{
-  initSR();
-
-  testkeysset_t keys;
-  generateKeyMaterial(DNSName("powerdns.com."), DNSSECKeeper::ECDSA256, DNSSECKeeper::DIGEST_SHA256, keys);
-
-  vector<DNSRecord> records;
-
-  sortedRecords_t recordContents;
-  vector<shared_ptr<const RRSIGRecordContent>> signatureContents;
-
-  ContentSigPair pair;
-  cspmap_t denialMap;
-
-  const DNSName requestedName("_ldap._tcp.a.b.c.d.powerdns.com.");
-  const DNSName zone("powerdns.com.");
-  /* Add NSEC3 for the closest encloser */
-  recordContents.clear();
-  signatureContents.clear();
-  records.clear();
-  addNSEC3UnhashedRecordToLW(DNSName("powerdns.com."), zone, "whatever", {QType::A, QType::TXT, QType::RRSIG, QType::NSEC}, 600, records);
-  recordContents.insert(records.at(0).getContent());
-  addRRSIG(keys, records, zone, 300);
-  signatureContents.push_back(getRR<RRSIGRecordContent>(records.at(1)));
-
-  pair.records = recordContents;
-  pair.signatures = signatureContents;
-  denialMap[std::pair(records.at(0).d_name, records.at(0).d_type)] = pair;
-
-  /* Add NSEC3 for the next closer */
-  recordContents.clear();
-  signatureContents.clear();
-  records.clear();
-  addNSEC3NarrowRecordToLW(DNSName("d.powerdns.com."), zone, {QType::A, QType::TXT, QType::RRSIG, QType::NSEC3}, 600, records);
-  recordContents.insert(records.at(0).getContent());
-  addRRSIG(keys, records, zone, 300);
-  signatureContents.push_back(getRR<RRSIGRecordContent>(records.at(1)));
-
-  pair.records = recordContents;
-  pair.signatures = signatureContents;
-  denialMap[std::pair(records.at(0).d_name, records.at(0).d_type)] = pair;
-
-  /* add wildcard denial */
-  recordContents.clear();
-  signatureContents.clear();
-  records.clear();
-  addNSEC3NarrowRecordToLW(DNSName("*.powerdns.com."), zone, {QType::A, QType::TXT, QType::RRSIG, QType::NSEC3}, 600, records);
-  recordContents.insert(records.at(0).getContent());
-  addRRSIG(keys, records, zone, 300);
-  signatureContents.push_back(getRR<RRSIGRecordContent>(records.at(1)));
-
-  pair.records = recordContents;
-  pair.signatures = signatureContents;
-  denialMap[std::pair(records.at(0).d_name, records.at(0).d_type)] = pair;
-
-  g_maxNSEC3sPerRecordToConsider = 10;
-  auto denialState = getDenial(denialMap, requestedName, QType::A, false, true);
-  g_maxNSEC3sPerRecordToConsider = 0;
-  BOOST_CHECK_EQUAL(denialState, dState::NXDOMAIN);
-}
-
 BOOST_AUTO_TEST_CASE(test_nsec3_insecure_delegation_denial)
 {
   initSR();
@@ -1581,20 +1522,6 @@ BOOST_AUTO_TEST_CASE(test_dnssec_validation_from_cache_secure_any)
   ret.clear();
   /* third one _does_ require validation */
   sr->setDNSSECValidationRequested(true);
-  MemRecursorCache::s_maxRRSetSize = 1;
-  BOOST_CHECK_THROW(sr->beginResolve(target, QType(QType::ANY), QClass::IN, ret), ImmediateServFailException);
-  // BOOST_CHECK_EQUAL(res, RCode::NoError);
-  // BOOST_CHECK_EQUAL(sr->getValidationState(), vState::Secure);
-  // BOOST_REQUIRE_EQUAL(ret.size(), 2U);
-  // for (const auto& record : ret) {
-  //   BOOST_CHECK(record.d_type == QType::A || record.d_type == QType::AAAA || record.d_type == QType::RRSIG);
-  // }
-  BOOST_CHECK_EQUAL(queriesCount, 2U);
-
-  ret.clear();
-  /* next one _does_ require validation */
-  MemRecursorCache::s_limitQTypeAny = false;
-  sr->setDNSSECValidationRequested(true);
   res = sr->beginResolve(target, QType(QType::ANY), QClass::IN, ret);
   BOOST_CHECK_EQUAL(res, RCode::NoError);
   BOOST_CHECK_EQUAL(sr->getValidationState(), vState::Secure);
